/*
 * page.cpp
 *
 *  Created on: 08.07.2010
 *      Author: vdv
 */

#include <errno.h>
#include <dirent.h>
#include <stdio.h>
#include <sys/stat.h>

#include "page.h"
#include "www_pages.h"

#define LOGO_TOOLTIP "logo"

#define WWW_MAIN "/main/"
#define WWW_OPTIONS "/options/"

bool reboot = false;

typedef struct
{
	const char* href;
	const char* text;
} pair;

static pair menu[] = {
	{
		WWW_MAIN,
		"Main"
	},
	{
		WWW_OPTIONS,
		"Options"
	},
};

/**
 * Generates html code for top menu
 * @param	menu_url - url to content
 * @return	html code for top menu with highlighting menu's name
 */
static std::string html_top_menu(const char* menu_url)
{
	std::string body = "<table class=\"topmenu\"><tr>";

	for( unsigned char i = 0; i < sizeof(menu) / sizeof(pair); i++ )
	{
		pair* p = &menu[i];
		body = body.append("<td");

		if( p->href != 0 )
		{
			const char* style = "";

			if( ::strcmp(p->href, menu_url) == 0 )
				style = " id=\"current\"";

			body = body.append(style).
					append("><a href=").
					append( p->href ).
					append(">").
					append( p->text ).
					append("</a>");
		}
		else
			body = body.append(">").append( p->text );

		body = body.append("</td>");
	}

	body = body.append("</tr></table>");

	return body;
}

static page_struct www_pages[] =
{
	{ WWW_MAIN, ::strlen(WWW_MAIN),
		www_main, www_main_post},
	{ WWW_OPTIONS, ::strlen(WWW_OPTIONS),
		www_options, www_options_post},
};
/**
 * Gets a page data for requested url.
 * @param	url - the url of a page.
 * @param	pages - a table of pages data.
 * @param	size - th size of the table.
 * @return	page_struct pointer to the page data.
 */
const page_struct* get_page_ptr(const char* url, page_struct* pages, size_t size)
{
	for( unsigned int i = 0; i < size; i++ )
	{
		const page_struct* p = &pages[i];
		if( ::strncmp(p->wwwpage, url, p->name_size) == 0 )
			return p;
	}
	return NULL;
}

/**
 * Creates POST processor, sets POST iterate function, POST completed function.
 * Return data sets into MHD_PostProcessor and into con_info.
 * @param	url - the url of a page
 * @param	connection - the MHD_connection
 * @param	con_info - the connection_info_struct
 */
void post_page_prepare(const char* url, struct MHD_Connection *connection, connection_info_struct* con_info)
{
	const page_struct* p = get_page_ptr( url, www_pages, sizeof(www_pages) / sizeof(page_struct) );
	if( p != NULL )
	{
		con_info->postprocessor =
			MHD_create_post_processor( connection, POSTBUFFERSIZE,
				(*p->post)( (url + p->name_size), con_info ), (void *) con_info );
	}
}
/**
 * Sends html code of the requested url
 * @param	url - the url of a page
 * @param	connection - the MHD_Connection
 * @return	response status, MHD_NO or MHD_YES
 */
int get_page(const char* url, struct MHD_Connection *connection)
{
	std::string page =
"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">" \
"<html>" \
" <head>" \
"  <title>" PAGE_TITLE "</title>" \
"  <link href=\"/style.css\" rel=\"stylesheet\" type=\"text/css\">" \
"  <meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">" \
" </head>" \
" <body>" \
" <table class=\"head\">" \
"  <tr>" \
"   <td><img src=\"/images/logo.png\" alt=\"" LOGO_TOOLTIP "\"></td>" \
"   <td width=\"100%\">";
	page.append( "Title" );
	page.append(
"</td>"\
"   <td>" \
"    <table>" \
"     <tr><td><a href=\"");
	page.append( url );
	page.append("?lang=ru\">Русский</a></td>" \
"         <td><a href=\"");
	page.append( url );
	page.append(
"?lang=en\">English</a></td></tr>" \
"    </table>" \
"   </td>" \
"  </tr>" \
" </table>");

	const page_struct* p = get_page_ptr( url, www_pages, sizeof(www_pages) / sizeof(page_struct) );
	if( p == NULL )// resource not found, return index.html
		p = &www_pages[0];

	page.append( html_top_menu(p->wwwpage) );
	std::string content = (*p->get)( (url + p->name_size) );

	page.append( content ).append("</body></html>");
	//return page;
	struct MHD_Response *response =
		MHD_create_response_from_data( page.length(), (void *) page.data(), MHD_NO, MHD_NO);
	if( !response )
		return MHD_NO;

	int ret = MHD_add_response_header( response, MHD_HTTP_HEADER_CONNECTION, "close" );
	if( ret != MHD_NO )
		ret = MHD_queue_response( connection, MHD_HTTP_OK, response );
	MHD_destroy_response( response );
	//
	if( reboot ) ::system("reboot");
	return ret;
}
