/*
 * web.cpp
 *
 *  Created on: 5.07.2010
 *      Author: vdv
 */

//TODO Set a handler for fatal errors. void MHD_set_panic_func(MHD PanicCallback cb, void *cls)
#include <string.h>

#include "platform.h"
#include "microhttpd.h"

#include "CWEBGUI.h"
#include "page.h"

#define PORT	8080
#define SERVER_DIRECTORY	"WWW"

#define DEFAULT_USER "user"
#define DEFAULT_PASSWORD "pass"
#define DEFAULT_USER_PASS_BASE64 "dXNlcjpwYXNz"

static struct MHD_Daemon *mhd_daemon;

/**
 * Check whether header correct authentification data
 * @param	connection - the current MHD_Connection
 * @return	true, if successful, false otherwise
 */
static bool is_authenticated( struct MHD_Connection *connection )
{
	const char* strbase = "Basic ";
	const char* headervalue = MHD_lookup_connection_value( connection,
						MHD_HEADER_KIND,
						MHD_HTTP_HEADER_AUTHORIZATION );

	if( NULL == headervalue )
		return false;
	if( 0 != strncmp(headervalue, strbase, ::strlen(strbase)) )
		return false;
	return (::strcmp(headervalue + ::strlen(strbase), DEFAULT_USER_PASS_BASE64) == 0);
}

/**
 * Callback function for sending resources
 *
 * @param cls - the pointer to FILE
 * @param pos - the position to send block of the data
 * @param buf - the buffer to read data
 * @param max - the size of the buffer
 * @return
 */
static int file_reader(void *cls, uint64_t pos, char *buf, int max)
{
	FILE *file = (FILE*)cls;

	::fseek( file, pos, SEEK_SET );
	return ::fread( buf, 1, max, file );
}
/**
 * Sends resource to recipient
 *
 * @param	connection - pointer to struct MHD_Connection
 * @param	resource - url, must be started with "/"
 * @param	mime - mime prefix, must be ended with "/"
 * @param	type - mime postfix
 * @return	response status
 */
static int send_resource(struct MHD_Connection* connection, const char* resource, const char* mime, const char* type)
{
	char* filename = (char*)::malloc( ::strlen(SERVER_DIRECTORY) + ::strlen(resource) + 1 );
	if( filename == NULL )
		return MHD_NO;
	::strcpy( filename, SERVER_DIRECTORY );
	::strcat( filename, resource );
	FILE* file = ::fopen( filename, "rb" );
	::free( filename );

	unsigned int status = MHD_HTTP_NOT_FOUND;
	struct MHD_Response* response;
	if( file != NULL )
	{
		struct stat buf;
		::fstat( ::fileno( file ), &buf );

		response = MHD_create_response_from_callback( buf.st_size,
				32 * 1024,// 32k page size
				&file_reader,
				file,
				(MHD_ContentReaderFreeCallback)&::fclose );

		if( response == NULL )
			return MHD_NO;

		char* mimetype = (char*)::malloc( ::strlen(mime) + ::strlen(type) + 1 );
		if( mimetype == NULL )
			return MHD_NO;
		::strcpy( mimetype, mime );
		::strcat( mimetype, type );

		int ret = MHD_add_response_header( response, MHD_HTTP_HEADER_CONTENT_TYPE, mimetype );
		::free( mimetype );
		if( ret == MHD_NO )
		{
			MHD_destroy_response( response );
			return ret;
		}
		status = MHD_HTTP_OK;
	} else
	{
		response = MHD_create_response_from_data( 0, NULL, MHD_NO, MHD_NO);
	}
	int ret = MHD_add_response_header( response, MHD_HTTP_HEADER_CONNECTION, "close" );
	if( ret != MHD_NO )
		ret = MHD_queue_response( connection, status, response );
	MHD_destroy_response( response );
	return ret;
}
/**
 * The MHD_RequestCompletedCallback.
 */
static void request_completed(void *cls, struct MHD_Connection *connection,
                   void **con_cls, enum MHD_RequestTerminationCode toe)
{
	printf( "request completed\r\n" );
	//
	struct connection_info_struct *con_info =
		(struct connection_info_struct *) *con_cls;

	if( NULL != con_info )
	{
		if( NULL != con_info->postprocessor )
			MHD_destroy_post_processor( con_info->postprocessor );
		
		if( NULL != con_info->file_ptr )
			::fclose( con_info->file_ptr );

		::free( con_info );
		*con_cls = NULL;
	}
}

/**
 * The MHD_AccessHandlerCallback
 */
static int answer_to_connection( void *cls,
						struct MHD_Connection *connection,
						const char *url,
						const char *method,
						const char *version,
						const char *upload_data,
						size_t *upload_data_size,
						void **con_cls )
{
	struct MHD_Response* response;
	int ret;
	unsigned int status = MHD_HTTP_NOT_FOUND;
	struct connection_info_struct *con_info;
	std::string page =
		"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">" \
		"<html><head><title>" PAGE_TITLE "</title>" \
		"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">" \
		"</head><body>";
		page.append( "Page not found" );
		page.append("</body></html>");;

	if( NULL == *con_cls )
	{
		//
		printf( "answer to connection, first call for url: %s\r\n", url );
		//
		if( (0 != ::strcmp(method, MHD_HTTP_METHOD_POST)) &&
			(0 != ::strcmp(method, MHD_HTTP_METHOD_GET)) )
		{// method is unknown
			return MHD_NO;
		}
		con_info = (connection_info_struct*)::malloc( sizeof(struct connection_info_struct) );
		if( NULL == con_info )
			return MHD_NO;

		con_info->file_ptr = NULL;
		con_info->postprocessor = NULL;
		con_info->post_completed = NULL;
		con_info->data = NULL;
		*con_cls = (void *) con_info;
		return MHD_YES;
	}
	//
	printf( "answer to connection, second call for url: %s\r\n", url );
	//
	bool auth = is_authenticated( connection );
	if( !auth )
	{
		page = "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">" \
			"<html><head><title>" PAGE_TITLE "</title>" \
			"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">" \
			"</head><body>";
		page.append( "Access denied" );
		page.append("</body></html>");
	}
	else
	{
		const char* type = ::strrchr( url, '.' );
		if( type != NULL )
		{
			type++;
			if( (::strcmp(type, "png") == 0) ||
				(::strcmp(type, "gif") == 0) ||
				(::strcmp(type, "bmp") == 0) ||
				(::strcmp(type, "tiff") == 0) ||
				(::strcmp(type, "ico") == 0) )
			{
				return send_resource( connection, url, "image/", type );
			} else if( (::strcmp(type, "jpg") == 0) ||
				(::strcmp(type, "jpeg") == 0) ||
				(::strcmp(type, "jpe") == 0) )
			{
				return send_resource( connection, url, "image/", "jpeg" );
			} else if( ::strcmp(type, "wav") == 0 )
			{
				return send_resource( connection, url, "audio/", "x-wav" );
			} else if( ::strcmp(type, "midi") == 0 )
			{
				return send_resource( connection, url, "audio/", type );
			} else if( ::strcmp(type, "mp3") == 0 )
			{
				return send_resource( connection, url, "audio/", "mpeg" );
			} else if( (::strcmp(type, "mpg") == 0) ||
				(::strcmp(type, "mpeg") == 0) ||
				(::strcmp(type, "mpe") == 0) )
			{
				return send_resource( connection, url, "video/", "mpeg" );
			} else if( (::strcmp(type, "qt") == 0) ||
				(::strcmp(type, ".mov") == 0) )
			{
				return send_resource( connection, url, "video/", "quicktime" );
			} else if( ::strcmp(type, "avi") == 0 )
			{
				return send_resource( connection, url, "video/", "x-msvideo" );
			} else if( ::strcmp(type, "txt") == 0 )
			{
				return send_resource( connection, url, "text/", "plain" );
			} else if( ::strcmp(type, "css") == 0 )
			{
				return send_resource( connection, url, "text/", type );
			} else if( ::strncmp(type, "htm", ::strlen("htm")) != 0 )
				return send_resource( connection, url, "text/", "plain" );
		}
		// if open directory or *.html, send page
		con_info = (connection_info_struct *)*con_cls;
		if( 0 == ::strcmp(method, MHD_HTTP_METHOD_POST) )
		{
			if( *upload_data_size != 0 )
			{
				if( NULL == con_info->postprocessor )
				{// first time call, create post processor
					post_page_prepare( url, connection, con_info );
					if( NULL == con_info->postprocessor )
						return MHD_NO;
				}
				MHD_post_process( con_info->postprocessor, upload_data, *upload_data_size );
				*upload_data_size = 0;
				return MHD_YES;
			}
			// post completed
			if( NULL != con_info->post_completed )
				(*(con_info->post_completed))( con_info );
			// return html page for POST. this is equal GET.
		}
		return get_page( url, connection );
	}

	response = MHD_create_response_from_data( page.length(),
											(void *)page.data(),
											MHD_NO,
											MHD_NO );
	if( response == NULL )
		return MHD_NO;

	if( !auth )
	{
		ret = MHD_add_response_header( response, MHD_HTTP_HEADER_WWW_AUTHENTICATE, "Basic realm=\"Secure area\"" );
		if( ret == MHD_NO )
		{
			MHD_destroy_response( response );
			return ret;
		}
		status = MHD_HTTP_UNAUTHORIZED;
	}

	ret = MHD_add_response_header( response, MHD_HTTP_HEADER_CONNECTION, "close" );
	if( ret != MHD_NO )
		ret = MHD_queue_response( connection, status, response );
	MHD_destroy_response( response );
	return ret;
}
/**
 * Starts the web gui interface
 * @param	arg - not used
 */
void* webgui_thread(void* arg)
{
	printf( "starting webgui on port %d\r\n", PORT );
	printf( "to access use USER: %s, PASSWORD: %s\r\n", DEFAULT_USER, DEFAULT_PASSWORD );
	mhd_daemon = MHD_start_daemon( MHD_USE_SELECT_INTERNALLY,
					PORT,
					NULL, NULL,
					&answer_to_connection, NULL,
					MHD_OPTION_NOTIFY_COMPLETED, request_completed, NULL,
					MHD_OPTION_CONNECTION_LIMIT, (unsigned int)2,// FD_SETSIZE - 4
					MHD_OPTION_PER_IP_CONNECTION_LIMIT, (unsigned int)2,// 0: unlimit
					MHD_OPTION_CONNECTION_TIMEOUT, (unsigned int)1800,// sec, 0: no timeout
					MHD_OPTION_END );

	if( mhd_daemon == NULL )
		printf( "webgui not started\r\n" );
	return NULL;
}
/**
 * Stops the web gui interface
 */
void webgui_thread_stop()
{
	MHD_stop_daemon( mhd_daemon );
	printf( "webgui is stopped\r\n" );
}
