/*
 * www_options.cpp
 *
 *  Created on: 08.07.2010
 *      Author: vdv
 */

#include "www_pages.h"

typedef struct
{
	const char* href;
	const char* text;
	const char* description;
} three;

static three options[] =
{
	{
		"op1.htm",
		"Options1",
		"Description"
	},
	{
		"op2.htm",
		"Options2",
		"Desciption"
	},
};

/**
 * Generates html code for left menu with highlighting option's name
 * @param	optionsname - option's name string
 * @return	html code for left menu
 */
static std::string html_menu_option(const char* optionsname)
{
	std::string body = "<table class=\"menu\">";

	for( unsigned int i = 0; i < sizeof(options) / sizeof(three); i++ )
	{
		three* v = &options[i];
		body.append("<tr><td");

		if( v->href != 0 )
		{
			if( ::strcmp(v->href, optionsname) == 0 )
				body.append(" id=\"current\"");

			body.append("><a href=/options/").
				append( v->href ).
				append(">").
				append( v->text ).
				append("</a></td></tr>");
		}
		else
			body.append(">").
				append( v->text ).
				append("</td></tr>");
	}
	body.append("</table>");
	return body;
}

static page_struct www_pages[] =
{
	{"op1.htm", ::strlen("op1.htm"),
		www_op1, www_op1_post},
	{"op2.htm", ::strlen("op2.htm"),
		www_op2, www_op2_post},
};

/**
 * Generates html code for menu "Options"
 * @param	url - the name of a page under menu "Options"
 * @return	html code menu "Options"
 */
std::string www_options(const char* url)
{
	const page_struct* p = get_page_ptr( url, www_pages, sizeof(www_pages) / sizeof(page_struct) );
	if( p != NULL )
	{
		std::string page = "<table><tr><td valign=\"top\">";
		page.append( html_menu_option(url) ).
			append("</td><td valign=\"top\" width=\"100%\">").
			append( (*p->get)(url + p->name_size) ).
			append("</td></tr></table>");
		return page;
	}
	// default content, index.html
	std::string body = "<table class=\"menu\">";
	for( unsigned int i = 0; i < sizeof(options) / sizeof(three); i++ )
	{
		three* v = &options[i];
		body.append("<tr><td><a href=/options/").
			append( v->href ).
			append(">").
			append( v->text ).
			append("</a></td>" \
				"<td width=\"100%\" id=\"past\">").
			append( v->description ).
			append("</td></tr>");
	}
	body.append("</table>");
	return body;
}

static int iterate_post(void *coninfo_cls, enum MHD_ValueKind kind, const char *key,
		const char *filename, const char *content_type,
		const char *transfer_encoding, const char *data, uint64_t off,
		size_t size)
{
	/* get data, if need
	my_data* system_data =
		(my_data*)((struct connection_info_struct *)coninfo_cls)->data;
	do anything
	*/
	return MHD_YES;
}

static void post_completed(connection_info_struct* con_info)
{
	/* free private data
	my_type* data = (my_type*)con_info->data;
	delete data;
	*/
}

MHD_PostDataIterator www_options_post(const char* url, connection_info_struct* con_info)
{
	const page_struct* p = get_page_ptr( url, www_pages, sizeof(www_pages) / sizeof(page_struct) );
	if( p != NULL )
	{
		return (*p->post)( (url + p->name_size), con_info );
	}
	con_info->post_completed = &post_completed;
	return &iterate_post;
}
